@@ -284,10 +284,7 @@ def show_hard_dependencies_in_cluster(node_id, cluster_path: Path) -> None:
             dependencies: set[str] = set()
             added_nodes = set([node_id])
             node_in_cluster = True
-    if not node_in_cluster:
-        raise ValueError(
-            "Desired node is not in the cluster. Note that namespacing this node is not currently supported."
-        )
+    assert node_in_cluster, "Desired node is not in the cluster. Note that namespacing this node is not currently supported."
     while len(added_nodes) > 0:
         moved_nodes: set[str] = set(added_nodes)
         dependencies = dependencies.union(moved_nodes)
@@ -331,6 +328,7 @@ def show_hard_dependencies_in_module(node_id: str, cluster_paths: list[Path]) ->
 cli.add_command(show_hard_dependencies_in_module)
 
 
+# TODO: don't print directly to facilitate testing
 @click.command()
 @click.argument("cluster_paths", type=click.Path(exists=True), nargs=-1)
 def check_learning_path(cluster_paths: list[Path]) -> None:
@@ -342,7 +340,7 @@ def check_learning_path(cluster_paths: list[Path]) -> None:
     comment_pattern = re.compile(r" *#.*")
     try:
         while True:
-            line = re.sub(comment_pattern, "", input().strip())
+            line = re.sub(comment_pattern, "", input()).strip()
             if line != "":
                 learning_path.append(line)
     except EOFError:
@@ -356,10 +354,7 @@ def check_learning_path(cluster_paths: list[Path]) -> None:
             matching_nodes = [
                 node for node in complete_graph.nodes if node.id == namespaced_node_id
             ]
-            if not matching_nodes:
-                raise ValueError(
-                    f"Node does not occur in the graph: {namespaced_node_id}"
-                )
+            assert len(matching_nodes) > 0, f"Node does not occur in the graph: {namespaced_node_id}"
             node = matching_nodes[0]
             namespaced_hard_dependencies = get_hard_dependencies_in_module(
                 namespaced_node_id, cluster_paths
@@ -380,14 +375,13 @@ def check_learning_path(cluster_paths: list[Path]) -> None:
                     print(
                         f"Node {index} ({namespaced_node_id}) has unmet dependency {dependency}."
                     )
+                # "self" here is the node currently being checked
                 self_and_dependents = nx.descendants(dependency_graph, node).union(
                     set([node])
                 )
                 is_motivated = False
                 for seen_node in seen_nodes:
-                    motivated_by_seen_node = nx.descendants(
-                        motivations_graph, seen_node
-                    )
+                    motivated_by_seen_node = nx.descendants_at_distance(motivations_graph, seen_node, 1)
                     if len(motivated_by_seen_node.intersection(self_and_dependents)):
                         is_motivated = True
                         break
@@ -460,10 +454,16 @@ def _retain_edges(graph, kind: Literal["dependency"] | Literal["motivation"]):
 @click.command()
 @click.argument("assignments_path", type=click.Path(exists=True), nargs=1)
 @click.argument("cluster_paths", type=click.Path(exists=True), nargs=-1)
-def generate_moodle_structure(assignments_path: Path, cluster_paths: list[Path]) -> None:
+@click.option("-m", "--model-path", type=click.Path(exists=True), required=False, help="Order nodes on this path linearly (and arbitrarily order nodes not on this path, but don't intersperse with those on the path).")
+def generate_moodle_structure(assignments_path: Path, cluster_paths: list[Path], model_path: Path | None) -> None:
     """Generates JSON data for the Moodle plugin.
     
     Assumes the module has been validated."""
+    def sentinel_index(lst, element, sentinel):
+        try:
+            return lst.index(element)
+        except ValueError:
+            return sentinel
     assignments_path = Path(assignments_path)
     module = Module(clusters=[])
     for path in cluster_paths:
@@ -472,6 +472,10 @@ def generate_moodle_structure(assignments_path: Path, cluster_paths: list[Path])
     dependency_graph = _retain_edges(dag, "dependency")
     motivation_graph = _retain_edges(dag, "motivation")
     assignments_path_start = assignments_path
+    comment_pattern = re.compile(r" *#.*")
+    if model_path:
+        with open(model_path, mode='r') as fh:
+            model_path_nodes = [re.sub(comment_pattern, "", line).strip() for line in fh.readlines() if line.strip()]
     collected_nodes = []
     for node in dag:
         current_node = dict()
@@ -495,6 +499,9 @@ def generate_moodle_structure(assignments_path: Path, cluster_paths: list[Path])
                     {"url": URL.url, "description": URL.description}
                 )
         collected_nodes.append(current_node)
+    if model_path:
+        # len... is to place all nodes not on path at the end
+        collected_nodes.sort(key=lambda n: sentinel_index(model_path_nodes,n['id'],len(model_path_nodes)))
     enablers: Dict = dict()
     for node in dag.nodes():
         self_and_dependents: Set[Node] = nx.descendants(dependency_graph, node).union(
@@ -525,4 +532,5 @@ cli.add_command(generate_moodle_structure)
 
 
 if __name__ == "__main__":
+    # writeschema()
     cli()
